import pydot


DEFAULT_NODE_ATTRS = {
    'color': 'cyan',
    'shape': 'box',
    'style': 'rounded',
    'fontname': 'palatino',
    'fontsize': 10,
    'penwidth': 2
}


def node_text(token):
    try:
        text = token._.plot['text']
    except:
        text = '{0} [{1}]\n({2} / {3})'.format(
            token.orth_,
            token.i,
            token.pos_,
            token.tag_
        )
    return text


def plot(doc):

    graph = pydot.Dot(graph_type='graph')

    # Add nodes to graph
    idx2node = {}
    for token in doc:
        try:
            plot_attrs = token._.plot
        except AttributeError:
            plot_attrs = {}
        for attr, val in DEFAULT_NODE_ATTRS.items():
            if attr not in plot_attrs:
                plot_attrs[attr] = val
        text = node_text(token)
        plot_attrs['name'] = text
        node = pydot.Node(**plot_attrs)
        idx2node[token.i] = node
        graph.add_node(node)

    '''Add edges'''
    for token in doc:
        if token.dep_ != 'ROOT':
            from_node = idx2node[token.i]
            to_node = idx2node[token.head.i]
            label = ' ' + token.dep_
            edge = pydot.Edge(
                to_node, from_node, label=label,
                fontsize=12
            )
            graph.add_edge(edge)

    png = graph.create_png()

    return png



import os
import en_core_web_sm
from spacy.tokens import Token
import visualise_spacy_tree

text = 'Forging involves the shaping of metal using localized compressive forces.'

nlp = en_core_web_sm.load()
doc = nlp(text)

example_plot_dir = 'visualise_spacy_tree/example_plots/'


def test_default_plot():
    plot = visualise_spacy_tree.plot(doc)
    with open(os.path.join(example_plot_dir, 'default_plot.png'), 'wb') as f:
        f.write(plot)


def test_custom_plot():
    Token.set_extension('plot', default={'color': 'aquamarine'})
    for token in doc:
        node_text = '{0} [{1}]\n({2} / {3})'.format(
                token.orth_,
                token.i,
                token.pos_,
                token.tag_
            )
        token._.plot['text'] = node_text
        if token.dep_ in ['ROOT', 'acl']:
            token._.plot['color'] = 'dodgerblue'
        if token.dep_ in ['nsubj', 'dobj']:
            token._.plot['color'] = 'deeppink1'
    plot = visualise_spacy_tree.plot(doc)
    with open(os.path.join(example_plot_dir, 'custom_plot.png'), 'wb') as f:
        f.write(plot)
